from __future__ import annotations

from sitri.strategy.single import SingleStrategy

from .mock import MockProvider


def test_get_equal(test_mock_provider: MockProvider, test_single_strategy: SingleStrategy) -> None:
    """test_get_equal.

    :param test_mock_provider:
    :param test_single_strategy:
    """
    assert test_mock_provider.get("test") == test_single_strategy.get("test")
    assert test_mock_provider.get("test.test_key1", path_mode=True) == test_single_strategy.get(
        "test.test_key1",
        path_mode=True,
    )



from __future__ import annotations

import typing as t

from sitri.settings.contrib.vault import VaultKVSettings


def test_metadata(vault_kv_config: VaultKVSettings) -> None:
    """test_metadata.

    :param vault_kv_config:
    :rtype: None
    """
    assert vault_kv_config.provider_code == "vault_kv"
    assert vault_kv_config._mount_point == "test"
    assert vault_kv_config._secret_path == "test"


def test_get_variable(monkeypatch: t.Any, vault_kv_config: VaultKVSettings) -> None:
    """test_get_variable.

    :param monkeypatch:
    :param vault_kv_config:
    :rtype: None
    """
    vault_kv_config._vault._env.update({"test": {"test": {"data": {}}}})

    vault_kv_config._vault._env["test"]["test"]["data"]["key1"] = "1"
    vault_kv_config._vault._env["test"]["test"]["data"]["key2"] = "2"

    assert vault_kv_config.get("key1") == "1"
    assert vault_kv_config.get("key2") == "2"


def test_get_variables_list(monkeypatch: t.Any, vault_kv_config: VaultKVSettings) -> None:
    """test_get_variables_list.

    :param monkeypatch:
    :param vault_kv_config:
    :rtype: None
    """
    vault_kv_config._vault._env.update({"test": {"test": {"data": {}}}})

    vault_kv_config._vault._env["test"]["test"]["data"]["key1"] = "1"
    vault_kv_config._vault._env["test"]["test"]["data"]["key2"] = "2"

    assert "key1" in vault_kv_config.keys()
    assert "key2" in vault_kv_config.keys()
    assert "key3" not in vault_kv_config.keys()

    monkeypatch.undo()



from __future__ import annotations

import typing as t

from sitri.providers.base import ConfigProvider
from sitri.strategy.base import BaseStrategy


class SingleStrategy(BaseStrategy):
    """SingleStrategy."""

    provider_code = "single"

    def __init__(self, data_provider: ConfigProvider) -> None:
        """__init__.

        :param data_provider:
        :type data_provider: ConfigProvider
        :rtype: None
        """
        self.provider = data_provider

    def get(self, *args: t.Any, **kwargs: t.Any) -> t.Any:
        """get.

        :param args:
        :param kwargs:
        """
        return self.provider.get(*args, **kwargs)

    def __getattribute__(self, item: str) -> t.Any:
        """__getattribute__.

        :param item:
        """
        try:
            return super().__getattribute__(item)

        except AttributeError:
            return self.provider.__getattribute__(item)



from __future__ import annotations

import os
import typing as t

import vedis


class VedisMock:
    """VedisMock."""

    _env = os.environ

    def get(self, key: str) -> bytes | None:
        """get.

        :param key:
        :type key: str
        :rtype: bytes | None
        """
        result = self._env.get(key)

        if result:
            return bytes(result, encoding="utf-8")
        else:
            return None

    def keys(self) -> t.List[bytes]:
        """keys.

        :rtype: t.List[bytes]
        """
        return [bytes(key, encoding="utf-8") for key in self._env.keys()]

    def __instancecheck__(self, instance: t.Any) -> bool:
        """__instancecheck__.

        :param instance:
        """
        return isinstance(instance, vedis.Vedis)

    def Hash(self, name: str) -> VedisMock:
        """Hash.

        :param name:
        :type name: str
        :rtype: "VedisMock"
        """
        return self



from __future__ import annotations

import configparser
import os
import typing as t

from sitri.providers.base import ConfigProvider


class IniConfigProvider(ConfigProvider):
    """Config provider for Initialization file (Ini)."""

    provider_code = "ini"

    def __init__(self, ini_path: str = "./config.ini", *args: t.Any, **kwargs: t.Any) -> None:
        """

        :param ini_path: path to ini file
        """
        super().__init__(*args, **kwargs)

        self.configparser = configparser.ConfigParser()

        with open(os.path.abspath(ini_path)) as f:
            self.configparser.read_file(f)

        self._sections = None

    @property
    def sections(self) -> t.List[str]:
        """sections."""
        if not self._sections:
            self._sections = list(self.configparser.keys())  # type: ignore

        return self._sections

    def get(self, key: str, section: str, **kwargs: t.Any) -> t.Any | None:  # type: ignore
        """Get value from ini file.

        :param key: key or path for search
        :param section: section of ini file
        """
        if section not in self.sections:
            return None

        return self.configparser[section].get(key)

    def keys(self, section: str, **kwargs: t.Any) -> t.List[str]:  # type: ignore
        """Get keys of section.

        :param section: section of ini file
        """
        if section not in self.sections:
            return []

        return list(self.configparser[section].keys())



from __future__ import annotations

import typing as t

from hvac.exceptions import VaultError
from pydantic import Field
from pydantic.main import BaseModel

from sitri.providers.contrib.json import JsonConfigProvider
from sitri.providers.contrib.vault import VaultKVConfigProvider
from sitri.settings.base import BaseLocalModeConfig, BaseLocalModeSettings


class VaultKVLocalProviderArgs(BaseModel):
    """VaultKVLocalProviderArgs."""

    json_path: str = Field(...)
    default_path_mode_state: bool = Field(default=True)


class VaultKVSettings(BaseLocalModeSettings):
    """VaultKVSettings."""

    @property
    def _local_provider_args(self) -> VaultKVLocalProviderArgs | t.Dict[str, t.Any]:
        """_local_provider_args."""
        args = self.__config__.local_provider_args
        if args:
            if isinstance(args, dict):
                args = VaultKVLocalProviderArgs(**args)

        return args

    @property
    def local_provider(self) -> JsonConfigProvider:
        """local_provider.

        :rtype: JsonConfigProvider
        """
        if not self.__config__.local_provider:
            args = self._local_provider_args

            if args:
                self.__config__.local_provider = JsonConfigProvider(
                    json_path=args.json_path,  # type: ignore
                    default_path_mode_state=args.default_path_mode_state,  # type: ignore
                    logger=self.__config__.local_provider_logger,
                )
            else:
                raise ValueError("Local provider arguments not found for local mode")

        return self.__config__.local_provider

    def _build_local(self, *args: t.Any, **kwargs: t.Any) -> t.Dict[str, t.Any]:
        """_build_local."""
        d: t.Dict[str, str | None] = {}

        for field in self.__fields__.values():
            value: str | None = None

            if self.__config__.local_mode_path_prefix:
                path = f"{self.__config__.local_mode_path_prefix}{self.local_provider.separator}{field.name}"

            else:
                path = field.name

            try:
                value = self.local_provider.get(key=path)
            except VaultError:
                self.local_provider.logger.error(f"Could not get local variable {path}")

            if field.is_complex():
                value = self._build_complex_value(value, path)

            if value is None and field.default is not None:
                value = field.default

            d[field.name] = value

        return d

    def _build_default(self, *args: t.Any, **kwargs: t.Any) -> t.Dict[str, t.Any]:
        """_build_default."""
        d: t.Dict[str, str | None] = {}

        provider = self.__config__.provider

        for field in self.__fields__.values():
            vault_val: str | None = None

            vault_secret_path = field.field_info.extra.get("vault_secret_path")
            vault_mount_point = field.field_info.extra.get("vault_mount_point")
            vault_secret_key = field.field_info.extra.get("vault_secret_key")

            if vault_secret_key is None:
                vault_secret_key = field.name

            try:
                vault_val = provider.get(
                    key=vault_secret_key,
                    secret_path=vault_secret_path if vault_secret_path else self.__config__.default_secret_path,
                    mount_point=vault_mount_point if vault_mount_point else self.__config__.default_mount_point,
                )
            except VaultError:
                provider.logger.warning(
                    f'Could not get secret "{vault_mount_point}/{vault_secret_path}:{vault_secret_key}"',
                )

            if field.is_complex():
                vault_val = self._build_complex_value(
                    vault_val,
                    f"{vault_mount_point}/{vault_secret_path}:{vault_secret_key}",
                )

            if vault_val is None and field.default is not None:
                vault_val = field.default

            d[field.name] = vault_val

        return d

    class VaultKVSettingsConfig(BaseLocalModeConfig):
        """VaultKVSettingsConfig."""

        provider: VaultKVConfigProvider
        default_secret_path: str | None = None
        default_mount_point: str | None = None

        local_mode: bool = False

        local_mode_path_prefix: str | None = None
        local_provider_args: VaultKVLocalProviderArgs | t.Dict[str, t.Any] | None

        local_provider: JsonConfigProvider | None = None

        local_provider_logger: t.Any | None = None

    __config__: VaultKVSettingsConfig



from __future__ import annotations

import typing as t

import hvac


class VaultClientMock:
    """VaultClientMock."""

    _env: t.Dict[str, t.Any] = {}

    def read_secret(self, mount_point: str, path: str) -> t.Any | None:
        """read_secret.

        :param mount_point:
        :type mount_point: str
        :param path:
        :type path: str
        :rtype: t.Any | None
        """

        return self._env.get(mount_point, {}).get(path)

    def __instancecheck__(self, instance: t.Any) -> bool:
        """__instancecheck__.

        :param instance:
        """
        return isinstance(instance, hvac.Client)

    def __getattribute__(self, item: str) -> t.Any:
        """__getattribute__.

        :param item:
        :rtype: t.Any
        """
        if item in ("secrets", "kv", "v1"):
            return self

        return super().__getattribute__(item)



"""
.. module:: config.providers
   :synopsis: Config Base
.. moduleauthor:: Aleksander Lavrov <github.com/egnod>
"""

from __future__ import annotations

import typing as t
from abc import ABC, abstractmethod


class PathModeStateProvider(ABC):
    """Base class for config providers with path_mode discovery."""

    _default_path_mode_state: bool = False

    def _get_path_mode_state(self, path_mode_param: bool | None) -> bool:
        """_get_path_mode_state.

        :param path_mode_param:
        :type path_mode_param: t.Optional[bool]
        :rtype: bool
        """
        return path_mode_param if path_mode_param is not None else self._default_path_mode_state

    @abstractmethod
    def get(self, key: str, path_mode: bool | None = None, **kwargs: t.Any) -> t.Any | None:
        """Get value from storage.

        :param key: key for get value
        :param path_mode: find value by path with separated key
        :param kwargs: additional arguments for providers
        """

    @abstractmethod
    def _get_by_path(self, path: str, separator: str) -> t.Any:
        """Retrieve value from a dictionary using a list of keys.

        :param path: string with separated keys
        """

    @abstractmethod
    def _get_by_key(self, key: str) -> t.Any:
        """Retrieve value from a dictionary using a key.

        :param key: key from json
        """

    @property
    @abstractmethod
    def data(self) -> t.Dict[str, t.Any]:
        """Retrieve data as dict.

        :param key: key from json
        """



from __future__ import annotations

import typing as t
from abc import ABC, abstractmethod
from pathlib import Path

from pydantic import BaseConfig as PydanticBaseConfig
from pydantic import BaseSettings as PydanticBaseSettings
from pydantic.env_settings import SettingsError
from pydantic.utils import deep_update

from sitri.providers.base import ConfigProvider


class BaseConfig(PydanticBaseConfig):
    """BaseConfig."""

    provider: t.Type[ConfigProvider] | ConfigProvider


class BaseLocalModeConfig(BaseConfig):
    """BaseLocalModeConfig."""

    provider: t.Type[ConfigProvider] | ConfigProvider

    local_mode: bool | None


class BaseSettings(ABC, PydanticBaseSettings):
    """BaseSettings."""

    def fill(self, call: t.Callable[[t.Any], t.Any], *args: t.Any, **kwargs: t.Any) -> t.Any:
        """fill.

        :param call:
        :type call: t.Callable
        """
        data = self.dict()
        return call(**data)

    def _build_complex_value(self, value: str | bytes | bytearray, path: str) -> t.Any:
        """_build_complex_value.

        :param value:
        :type value: Union[str, bytes, bytearray]
        :param path:
        :type path: str
        """
        if isinstance(value, str) or isinstance(value, bytes) or isinstance(value, bytearray):
            try:
                value = self.__config__.json_loads(value)
            except ValueError as e:
                raise SettingsError(f"Error parsing for variable {path}") from e

        return value

    def _build_values(
        self,
        init_kwargs: t.Dict[str, t.Any],
        _env_file: Path | str | None = None,
        _env_file_encoding: str | None = None,
        _env_nested_delimiter: str | None = None,
        _secrets_dir: Path | str | None = None,
    ) -> t.Dict[str, t.Any]:
        """_build_values.

        :param init_kwargs:
        :type init_kwargs: t.Dict[str, t.Any]
        :param _env_file:
        :type _env_file: Union[Path, str, None]
        :param _env_file_encoding:
        :type _env_file_encoding: Optional[str]
        :param _env_nested_delimiter:
        :type _env_nested_delimiter: Optional[str]
        :param _secrets_dir:
        :type _secrets_dir: Union[Path, str, None]
        :rtype: t.Dict[str, t.Any]
        """
        return deep_update(
            deep_update(self._build_default()),
            init_kwargs,
        )

    @abstractmethod
    def _build_default(self, *args: t.Any, **kwargs: t.Any) -> t.Dict[str, t.Any]:
        """_build_default.

        :param args:
        :param kwargs:
        :rtype: t.Dict[str, t.Any]
        """
        pass

    __config__: t.Type[BaseConfig]


class BaseLocalModeSettings(BaseSettings):
    """BaseLocalModeSettings."""

    @property
    @abstractmethod
    def local_provider(self) -> t.Type[ConfigProvider] | ConfigProvider:
        """local_provider.

        :rtype: Union[t.Type[ConfigProvider], ConfigProvider]
        """
        pass

    @abstractmethod
    def _build_local(self, *args: t.Any, **kwargs: t.Any) -> t.Dict[str, t.Any]:
        """_build_local.

        :param args:
        :param kwargs:
        :rtype: t.Dict[str, t.Any]
        """
        pass

    def _build_values(
        self,
        init_kwargs: t.Dict[str, t.Any],
        _env_file: Path | str | None = None,
        _env_file_encoding: str | None = None,
        _env_nested_delimiter: str | None = None,
        _secrets_dir: Path | str | None = None,
    ) -> t.Dict[str, t.Any]:
        """_build_values.

        :param init_kwargs:
        :type init_kwargs: t.Dict[str, t.Any]
        :param _env_file:
        :type _env_file: Union[Path, str, None]
        :param _env_file_encoding:
        :type _env_file_encoding: Optional[str]
        :param _env_nested_delimiter:
        :type _env_nested_delimiter: Optional[str]
        :param _secrets_dir:
        :type _secrets_dir: Union[Path, str, None]
        :rtype: t.Dict[str, t.Any]
        """
        if not self.__config__.local_mode:
            return deep_update(
                deep_update(self._build_default()),
                init_kwargs,
            )
        else:
            return deep_update(deep_update(self._build_local()), init_kwargs)

    __config__: t.Type[BaseLocalModeConfig]



""".. module:: __init__

.. moduleauthor:: Aleksander Lavrov <github.com/egnod>
"""

from sitri.configurator import SitriProviderConfigurator

Sitri = SitriProviderConfigurator  # noqa


__version__ = "2.2.11"



from __future__ import annotations

import pytest

from sitri.providers.contrib.ini import IniConfigProvider


def test_no_file_error() -> None:
    """test_no_file_error."""
    with pytest.raises(FileNotFoundError):
        IniConfigProvider(ini_path="not_exists.ini")


def test_metadata(ini_config: IniConfigProvider) -> None:
    """test_metadata.

    :param ini_config:
    :rtype: None
    """
    assert ini_config.provider_code == "ini"


def test_sections(ini_config: IniConfigProvider) -> None:
    """test_sections.

    :param ini_config:
    """
    assert isinstance(ini_config.sections, list)
    assert sorted(ini_config.sections) == ["DEFAULT", "test_a", "test_b"]


def test_keys(ini_config: IniConfigProvider) -> None:
    """test_keys.

    :param ini_config:
    """
    assert ini_config.keys("test_a") == ["hello"]
    assert ini_config.keys("test_b") == ["str", "int"]
    assert ini_config.keys("not_exist") == []


def test_get(ini_config: IniConfigProvider) -> None:
    """test_get.

    :param ini_config:
    """
    assert ini_config.get(section="test_a", key="hello") == "world"
    assert ini_config.get(section="test_a", key="not_exist") is None
    assert ini_config.get(section="test_b", key="str") == "str"
    assert ini_config.get(section="test_b", key="int") == "1"
    assert ini_config.get(section="not_exist", key="int") is None



from __future__ import annotations

import typing as t

import pytest

from sitri.providers.contrib.vedis import VedisConfigProvider

from .mock import VedisMock


@pytest.fixture(scope="module")
def vedis_connection() -> t.Callable[[], VedisMock]:
    """vedis_connection.

    :rtype: t.Callable
    """
    return lambda: VedisMock()


@pytest.fixture(scope="module")
def vedis_config(vedis_connection: t.Callable[[], VedisMock]) -> VedisConfigProvider:
    """vedis_config.

    :param vedis_connection:
    :rtype: VedisConfigProvider
    """
    return VedisConfigProvider(vedis_connector=vedis_connection, hash_name="test")



from __future__ import annotations

import typing as t

import pytest
from pydantic import BaseModel, Field

from sitri.providers.contrib.yaml import YamlConfigProvider
from sitri.settings.contrib.yaml import YamlSettings

T = t.TypeVar("T", bound=YamlSettings)


@pytest.fixture(scope="module")
def path_to_yaml() -> str:
    """path_to_yaml.

    :rtype: str
    """
    return "tests/unit/settings/yaml/data.yaml"


@pytest.fixture(scope="module")
def yaml_config(path_to_yaml: str) -> YamlConfigProvider:
    """yaml_config.

    :param path_to_yaml:
    :rtype: YamlConfigProvider
    """
    return YamlConfigProvider(yaml_path=path_to_yaml)  # noqa


@pytest.fixture(scope="module")
def yaml_settings_empty(yaml_config: YamlConfigProvider) -> t.Type[T]:
    """yaml_settings_empty.

    :param yaml_config:
    """

    class TestSettings(YamlSettings):
        """TestSettings."""

        class Config(YamlSettings.YamlSettingsConfig):
            """Config."""

            provider = yaml_config

    return TestSettings


@pytest.fixture(scope="module")
def yaml_settings() -> t.Callable[[YamlConfigProvider], t.Type[T]]:
    """yaml_settings."""

    def wrapper(provider_instance: YamlConfigProvider) -> t.Type[T]:
        """wrapper.

        :param provider_instance:
        """

        class TestSettings(YamlSettings):
            """TestSettings."""

            key1: str = Field(...)
            key2: str = Field(...)
            key3: str = Field(...)

            class Config(YamlSettings.YamlSettingsConfig):
                """Config."""

                provider = provider_instance

        return TestSettings

    return wrapper


@pytest.fixture(scope="module")
def yaml_settings_raise() -> t.Callable[[YamlConfigProvider], t.Type[T]]:
    """yaml_settings_raise."""

    def wrapper(provider_instance: YamlConfigProvider) -> t.Type[T]:
        """wrapper.

        :param provider_instance:
        """

        class TestSettings(YamlSettings):
            """TestSettings."""

            key0: str = Field(...)

            class Config(YamlSettings.YamlSettingsConfig):
                """Config."""

                provider = provider_instance

        return TestSettings

    return wrapper


@pytest.fixture(scope="module")
def yaml_settings_complex() -> t.Callable[[YamlConfigProvider], t.Type[T]]:
    """yaml_settings_complex."""

    def wrapper(provider_instance: YamlConfigProvider) -> t.Type[T]:
        """wrapper.

        :param provider_instance:
        """

        class Key4Model(BaseModel):
            """Key4Model."""

            test_key4_field: str = Field(...)

        class TestSettings(YamlSettings):
            """TestSettings."""

            key4: Key4Model = Field(...)

            class Config(YamlSettings.YamlSettingsConfig):
                """Config."""

                provider = provider_instance

        TestSettings.update_forward_refs(Key4Model=Key4Model)

        return TestSettings

    return wrapper


@pytest.fixture(scope="module")
def yaml_settings_complex_raise() -> t.Callable[[YamlConfigProvider], t.Type[T]]:
    """yaml_settings_complex_raise."""

    def wrapper(provider_instance: YamlConfigProvider) -> t.Type[T]:
        """wrapper.

        :param provider_instance:
        """

        class Key5Model(BaseModel):
            """Key5Model."""

            test: str = Field(...)

        class TestSettings(YamlSettings):
            """TestSettings."""

            key5: Key5Model = Field(...)

            class Config(YamlSettings.YamlSettingsConfig):
                """Config."""

                provider = provider_instance

        TestSettings.update_forward_refs(Key5Model=Key5Model)

        return TestSettings

    return wrapper



"""
.. module:: configurator
   :synopsis: General Sitri class
.. moduleauthor:: Aleksander Lavrov <github.com/egnod>
"""
from __future__ import annotations

import typing as t

from sitri.logger import get_default_logger
from sitri.providers.base import ConfigProvider
from sitri.strategy.base import BaseStrategy
from sitri.strategy.single import SingleStrategy


class SitriProviderConfigurator:
    """Class for unite config provider."""

    def __init__(self, config_provider: ConfigProvider | BaseStrategy, logger: t.Any | None = None) -> None:
        """
        :param config_provider: object of config provider
        """

        self.config = None

        if not logger:
            logger = get_default_logger()

        self.logger = logger

        if isinstance(config_provider, BaseStrategy):
            self.config = config_provider

        elif config_provider:
            self.config = SingleStrategy(config_provider)

    def get(self, key: str, default: None = None, **kwargs: t.Any) -> str | None:
        """Get value from config provider.

        :param key: key for config provider
        :param default: if provider return None
        """
        if not self.config:
            self.logger.info("No config provider")
            return None

        variable = self.config.get(key, **kwargs)

        return variable if variable else default



from __future__ import annotations

import pytest

from sitri import Sitri
from sitri.providers.contrib.system import SystemConfigProvider


@pytest.fixture(scope="module")
def test_sitri() -> Sitri:
    """test_sitri."""
    return Sitri(config_provider=SystemConfigProvider(prefix="test"))



from __future__ import annotations

import typing as t

from sitri.providers.contrib.system import SystemConfigProvider


def test_metadata(system_config: SystemConfigProvider) -> None:
    """test_metadata.

    :param system_config:
    :rtype: None
    """
    assert system_config.provider_code == "system"
    assert system_config._prefix == "TEST"


def test_prefixize(system_config: SystemConfigProvider) -> None:
    """test_prefixize.

    :param system_config:
    :rtype: None
    """
    assert system_config.prefixize("key1") == "TEST_KEY1"
    assert system_config.unprefixize("TEST_KEY1") == "key1"


def test_get_variable(monkeypatch: t.Any, system_config: SystemConfigProvider) -> None:
    """test_get_variable.

    :param monkeypatch:
    :param system_config:
    :rtype: None
    """
    monkeypatch.setenv("TEST_KEY1", "1")
    monkeypatch.setenv("TEST_KEY2", "2")

    assert system_config.get("key1") == "1"
    assert system_config.get("key2") == "2"

    monkeypatch.undo()


def test_get_variables_list(monkeypatch: t.Any, system_config: SystemConfigProvider) -> None:
    """test_get_variables_list.

    :param monkeypatch:
    :param system_config:
    :rtype: None
    """
    monkeypatch.setenv("TEST_KEY1", "1")
    monkeypatch.setenv("TEST_KEY2", "2")

    monkeypatch.setenv("TEZT_T1", "1")
    monkeypatch.setenv("TEZT_T2", "2")

    assert "key1" in system_config.keys()
    assert "key2" in system_config.keys()
    assert "t1" not in system_config.keys()
    assert "t2" not in system_config.keys()

    monkeypatch.undo()



from __future__ import annotations

import typing as t

from sitri.providers.contrib.yaml import YamlConfigProvider
from sitri.settings.base import BaseConfig, BaseSettings


class YamlSettings(BaseSettings):
    """YamlSettings."""

    def _build_default(self, *args: t.Any, **kwargs: t.Any) -> t.Dict[str, t.Any]:
        """_build_default."""
        d: t.Dict[str, str | None] = {}

        provider = self.__config__.provider

        for field in self.__fields__.values():
            key_name: str | None = field.field_info.extra.get("yaml_key_name")
            path_prefix: str | None = field.field_info.extra.get("yaml_path_prefix")

            if key_name is None:
                key_name = field.name

            if path_prefix is not None:
                key_name = f"{path_prefix}{provider.separator}{key_name}"

            elif self.__config__.default_path_prefix is not None:
                key_name = f"{self.__config__.default_path_prefix}{provider.separator}{key_name}"

            value: str | None = provider.get(key_name)

            if field.is_complex():
                value = self._build_complex_value(value, key_name)

            if value is None and field.default is not None:
                value = field.default

            d[field.name] = value

        return d

    class YamlSettingsConfig(BaseConfig):
        """YamlSettingsConfig."""

        provider: YamlConfigProvider

        default_path_prefix: str | None = None

    __config__: YamlSettingsConfig



from __future__ import annotations

import pytest

from sitri.strategy.single import SingleStrategy

from .mock import MockProvider


@pytest.fixture(scope="module")
def test_mock_provider() -> MockProvider:
    """test_mock_provider."""
    return MockProvider(json_path="tests/unit/strategy/single/data.json")


@pytest.fixture
def test_single_strategy(test_mock_provider: MockProvider) -> SingleStrategy:
    """test_single_strategy.

    :param test_mock_provider:
    """
    return SingleStrategy(test_mock_provider)



from __future__ import annotations

import logging


def get_default_logger(name: str | None = None) -> logging.Logger:
    """get_default_logger.

    :param name:
    :type name: t.Optional[str]
    :rtype: logging.Logger
    """
    logging.basicConfig(
        level=logging.INFO,
        format="%(asctime)s - [%(levelname)s] - %(name)s - (%(filename)s).%(funcName)s(%(lineno)d) - %(message)s",
    )

    return logging.getLogger(name or "sitri.default")



from __future__ import annotations

import os
import typing as t

import tomli as toml

from sitri.providers.base import ConfigProvider, PathModeStateProvider


class TomlConfigProvider(PathModeStateProvider, ConfigProvider):
    """Config provider for TOML."""

    provider_code = "toml"

    def __init__(
        self,
        toml_path: str = "./data.toml",
        toml_data: str | None = None,
        default_separator: str = ".",
        found_file_error: bool = True,
        default_path_mode_state: bool = False,
        *args: t.Any,
        **kwargs: t.Any,
    ) -> None:
        """

        :param toml_path: path to toml file
        :param toml_data: toml data in string
        :param default_separator: default value separator for path-mode
        :param found_file_error: if true no file not found error raise on toml.load
        :param default_path_mode_state: default state for path mode on get value by key
        """
        super().__init__(*args, **kwargs)

        if not toml_data:
            self._toml = self._get_toml_from_file(toml_path, found_file_error)
        else:
            self._toml = toml.loads(toml_data)

        self.separator = default_separator
        self._default_path_mode_state = default_path_mode_state

    @staticmethod
    def _get_toml_from_file(toml_path: str, found_file_error: bool) -> t.Dict[str, t.Any]:
        """_get_toml_from_file.

        :param toml_path:
        :type toml_path: str
        :param found_file_error:
        :type found_file_error: bool
        """
        try:
            with open(os.path.abspath(toml_path), "rb") as f:
                data = toml.load(f)

            return data

        except FileNotFoundError:
            if not found_file_error:
                return {}
            else:
                raise

    def _get_by_path(self, path: str, separator: str) -> t.Any:
        """Retrieve value from a dictionary using a list of keys.

        :param path: string with separated keys
        """
        dict_local = self._toml.copy()
        keys = path.split(separator)

        for key in keys:
            try:
                dict_local = dict_local[int(key)] if key.isdigit() else dict_local[key]  # type: ignore
            except Exception:
                if key not in dict_local:
                    return None

                dict_local = dict_local[key]
        return dict_local

    def _get_by_key(self, key: str) -> t.Any:
        """Retrieve value from a dictionary using a key.

        :param key: key from json
        """

        if key in self._toml:
            return self._toml[key]
        else:
            return None

    def get(self, key: str, path_mode: bool | None = None, separator: str = None, **kwargs: t.Any) -> t.Any | None:
        """Get value from json.

        :param key: key or path for search
        :param path_mode: boolean mode switcher
        :param separator: separator for path keys in path mode
        """

        separator = separator if separator else self.separator

        if self._get_path_mode_state(path_mode):
            return self._get_by_path(key, separator=separator)

        return self._get_by_key(key)

    def keys(self, path_mode: bool = False, separator: str = None, **kwargs: t.Any) -> t.List[str]:
        """Keys in json.

        :param path_mode: [future] path mode for keys list
        :param separator: [future] separators for keys in path mode
        """
        # TODO: implemented path-mode for keys list

        if not path_mode:
            return list(self._toml.keys())
        else:
            raise NotImplementedError("Path-mode not implemented!")

    @property
    def data(self) -> t.Dict[str, t.Any]:
        """Retrieve data as dict."""

        return self._toml



from __future__ import annotations

import pytest

from sitri.providers.contrib.toml import TomlConfigProvider


def test_no_file_error() -> None:
    """test_no_file_error."""
    with pytest.raises(FileNotFoundError):
        TomlConfigProvider(toml_path="_data.toml")

    TomlConfigProvider(toml_path="_data.toml", found_file_error=False)


@pytest.mark.parametrize(
    "toml_config_obj",
    [pytest.lazy_fixture("toml_config"), pytest.lazy_fixture("toml_data_config")],
)
def test_metadata(toml_config_obj: TomlConfigProvider) -> None:
    """test_metadata.

    :param toml_config_obj:
    :rtype: None
    """
    assert toml_config_obj.provider_code == "toml"
    assert toml_config_obj.separator == "/"


@pytest.mark.parametrize(
    "toml_config_obj",
    [pytest.lazy_fixture("toml_config"), pytest.lazy_fixture("toml_data_config")],
)
def test_get_by_other(toml_config_obj: TomlConfigProvider) -> None:
    """test_get_by_other.

    :param toml_config_obj:
    :rtype: None
    """
    assert isinstance(toml_config_obj._get_by_path("test", separator=toml_config_obj.separator), dict)
    assert toml_config_obj._get_by_key("test.test_key2") is None
    assert toml_config_obj._get_by_path("test.test_key2", separator=toml_config_obj.separator) is None
    assert isinstance(toml_config_obj._get_by_key("test"), dict)
    assert toml_config_obj._get_by_path("test", separator=toml_config_obj.separator) == toml_config_obj._get_by_key(
        "test",
    )
    assert toml_config_obj._get_by_path(
        "test/test_key2",
        separator=toml_config_obj.separator,
    ) == toml_config_obj._get_by_key("test").get("test_key2")


@pytest.mark.parametrize(
    "toml_config_obj",
    [pytest.lazy_fixture("toml_config"), pytest.lazy_fixture("toml_data_config")],
)
def test_get(toml_config_obj: TomlConfigProvider) -> None:
    """test_get.

    :param toml_config_obj:
    """
    assert isinstance(toml_config_obj.get("test"), dict)
    assert isinstance(toml_config_obj.get("test", path_mode=True), dict)
    assert toml_config_obj.get("test", path_mode=True) == toml_config_obj.get("test")
    assert toml_config_obj.get("test/test_key2", path_mode=True) and toml_config_obj.get(
        "test/test_key2",
        path_mode=True,
    ) == toml_config_obj.get("test").get("test_key2")
    assert toml_config_obj.get("test.test_key2", separator=".", path_mode=True) == toml_config_obj.get(
        "test/test_key2",
        path_mode=True,
    )
    assert not toml_config_obj.get("test/test_key2")


@pytest.mark.parametrize(
    "toml_config_obj",
    [pytest.lazy_fixture("toml_config"), pytest.lazy_fixture("toml_data_config")],
)
def test_keys(toml_config_obj: TomlConfigProvider) -> None:
    """test_keys.

    :param toml_config_obj:
    """
    assert "test" in toml_config_obj.keys()

    with pytest.raises(NotImplementedError):
        toml_config_obj.keys(path_mode=True)



from __future__ import annotations

import pytest
import tomli_w

from sitri.providers.contrib.toml import TomlConfigProvider


@pytest.fixture(scope="module")
def toml_data() -> str:
    """toml_data.

    :rtype: str
    """
    data = {"test": {"test_key1": "1", "test_key2": "2", "test_key3": "3"}}

    return tomli_w.dumps(data)


@pytest.fixture(scope="module")
def path_to_toml() -> str:
    """path_to_toml.

    :rtype: str
    """
    return "tests/unit/configs/toml/data.toml"


@pytest.fixture(scope="module")
def toml_data_config(toml_data: str) -> TomlConfigProvider:
    """toml_data_config.

    :param toml_data:
    :rtype: TomlConfigProvider
    """

    return TomlConfigProvider(toml_data=toml_data, default_separator="/")


@pytest.fixture(scope="module")
def toml_config(path_to_toml: str) -> TomlConfigProvider:
    """toml_config.

    :param path_to_toml:
    :rtype: TomlConfigProvider
    """
    return TomlConfigProvider(toml_path=path_to_toml, default_separator="/")



from __future__ import annotations

from _pytest.monkeypatch import MonkeyPatch

from sitri.providers.contrib.vedis import VedisConfigProvider


def test_metadata(vedis_config: VedisConfigProvider) -> None:
    """test_metadata.

    :param vedis_config:
    :rtype: None
    """
    assert vedis_config.provider_code == "vedis"
    assert vedis_config._hash_name == "test"


def test_get_variable(monkeypatch: MonkeyPatch, vedis_config: VedisConfigProvider) -> None:
    """test_get_variable.

    :param monkeypatch:
    :param vedis_config:
    :rtype: None
    """
    monkeypatch.setenv("key1", "1")
    monkeypatch.setenv("key2", "2")

    assert vedis_config.get("key1") == "1"
    assert vedis_config.get("key2") == "2"

    monkeypatch.undo()


def test_get_variables_list(monkeypatch: MonkeyPatch, vedis_config: VedisConfigProvider) -> None:
    """test_get_variables_list.

    :param monkeypatch:
    :param vedis_config:
    :rtype: None
    """
    monkeypatch.setenv("key1", "1")
    monkeypatch.setenv("key2", "2")

    assert "key1" in vedis_config.keys()
    assert "key2" in vedis_config.keys()
    assert "kZ1" not in vedis_config.keys()
    assert "kZ2" not in vedis_config.keys()

    monkeypatch.undo()



from __future__ import annotations

import typing as t

import pytest

from sitri.providers.contrib.vault import VaultKVConfigProvider

from .mock import VaultClientMock


@pytest.fixture(scope="module")
def vault_connection() -> t.Callable[[], VaultClientMock]:
    """vault_connection.

    :rtype: t.Callable
    """
    return lambda: VaultClientMock()


@pytest.fixture(scope="module")
def vault_kv_config(vault_connection: t.Callable[[], VaultClientMock]) -> VaultKVConfigProvider:
    """vault_kv_config.

    :param vault_connection:
    :rtype: VaultKVConfigProvider
    """
    return VaultKVConfigProvider(vault_connector=vault_connection, mount_point="test", secret_path="test")  # noqa



from __future__ import annotations

import typing as t

from sitri.providers.contrib.redis import RedisConfigProvider


def test_metadata(redis_config: RedisConfigProvider) -> None:
    """test_metadata.

    :param redis_config:
    :rtype: None
    """
    assert redis_config.provider_code == "redis"
    assert redis_config._prefix == "TEST"


def test_prefixize(redis_config: RedisConfigProvider) -> None:
    """test_prefixize.

    :param redis_config:
    :rtype: None
    """
    assert redis_config.prefixize("key1") == "TEST_KEY1"
    assert redis_config.unprefixize("TEST_KEY1") == "key1"


def test_get_variable(monkeypatch: t.Any, redis_config: RedisConfigProvider) -> None:
    """test_get_variable.

    :param monkeypatch:
    :param redis_config:
    :rtype: None
    """
    monkeypatch.setenv("TEST_KEY1", "1")
    monkeypatch.setenv("TEST_KEY2", "2")

    assert redis_config.get("key1") == "1"
    assert redis_config.get("key2") == "2"

    monkeypatch.undo()


def test_get_variables_list(monkeypatch: t.Any, redis_config: RedisConfigProvider) -> None:
    """test_get_variables_list.

    :param monkeypatch:
    :param redis_config:
    :rtype: None
    """
    monkeypatch.setenv("TEST_KEY1", "1")
    monkeypatch.setenv("TEST_KEY2", "2")

    monkeypatch.setenv("TEZT_T1", "1")
    monkeypatch.setenv("TEZT_T2", "2")

    assert "key1" in redis_config.keys()
    assert "key2" in redis_config.keys()
    assert "t1" not in redis_config.keys()
    assert "t2" not in redis_config.keys()

    monkeypatch.undo()



from __future__ import annotations

import typing as t

import pytest
from pydantic.env_settings import SettingsError
from pydantic.error_wrappers import ValidationError

from sitri.providers.contrib.vault import VaultKVConfigProvider
from sitri.settings.contrib.vault import VaultKVSettings


def test_metadata(vault_kv_settings_empty: VaultKVSettings) -> None:
    """test_metadata.

    :param vault_kv_settings_empty:
    :rtype: None
    """
    assert vault_kv_settings_empty.Config.provider.provider_code == "vault_kv"


def test_get_variable(
    monkeypatch: t.Any,
    vault_kv_config: VaultKVConfigProvider,
    vault_kv_settings: t.Callable[[VaultKVConfigProvider], VaultKVSettings],
    vault_kv_settings_vault_raise: t.Callable[[VaultKVConfigProvider], VaultKVSettings],
    vault_kv_settings_complex_raise: t.Callable[[VaultKVConfigProvider], VaultKVSettings],
    vault_kv_settings_complex: t.Callable[[VaultKVConfigProvider], VaultKVSettings],
) -> None:
    """test_get_variable.

    :param monkeypatch:
    :param vault_kv_config:
    :param vault_kv_settings:
    :param vault_kv_settings_vault_raise:
    :param vault_kv_settings_complex_raise:
    :param vault_kv_settings_complex:
    :rtype: None
    """
    vault_kv_config._vault._env.update({"test": {"test": {"data": {}}}})

    vault_kv_config._vault._env["test"]["test"]["data"]["key1"] = "1"
    vault_kv_config._vault._env["test"]["test"]["data"]["key2"] = "2"
    vault_kv_config._vault._env["test"]["test"]["data"]["key4"] = r'{"test" "test"}'
    vault_kv_config._vault._env["test"]["test"]["data"]["key0"] = r'{"test": "test"}'

    test_settings = vault_kv_settings(provider_instance=vault_kv_config)()
    assert test_settings.key1 == "1"
    assert test_settings.key2 == "2"

    test_settings_complex = vault_kv_settings_complex(provider_instance=vault_kv_config)()
    assert test_settings_complex.key0 == test_settings_complex.__config__.json_loads(
        vault_kv_config._vault._env["test"]["test"]["data"]["key0"],
    )

    with pytest.raises(ValidationError):
        vault_kv_settings_vault_raise(provider_instance=vault_kv_config)()

    with pytest.raises(SettingsError):
        vault_kv_settings_complex_raise(provider_instance=vault_kv_config)()


def test_get_variable_local_mode(
    vault_kv_config: VaultKVConfigProvider,
    vault_kv_local_mode: t.Callable[[VaultKVConfigProvider], VaultKVSettings],
) -> None:
    """test_get_variable_local_mode.

    :param vault_kv_config:
    :param vault_kv_local_mode:
    """
    test_settings = vault_kv_local_mode(provider_instance=vault_kv_config)()

    assert test_settings.key1 == "1"
    assert test_settings.key2 == "2"
    assert test_settings.key3 == "3"

    assert "test" in test_settings.key4.dict()
    assert test_settings.key4.test == "test"



from __future__ import annotations

from sitri.strategy.index_priority import IndexPriorityStrategy

from .mock import MockProvider


def test_get_equal(
    test_index_priority_strategy: IndexPriorityStrategy,
    test_mock_provider1: MockProvider,
    test_mock_provider2: MockProvider,
) -> None:
    """test_get_equal.

    :param test_index_priority_strategy:
    :param test_mock_provider1:
    :param test_mock_provider2:
    """
    assert test_mock_provider1.get("test") == test_index_priority_strategy.get("test")
    assert test_mock_provider1.get("test.test_key1", path_mode=True) == test_index_priority_strategy.get(
        "test.test_key1",
        path_mode=True,
    )
    assert test_mock_provider2.get("test.test_key4", path_mode=True) == test_index_priority_strategy.get(
        "test.test_key4",
        path_mode=True,
    )


def test_get(
    test_index_priority_strategy: IndexPriorityStrategy,
    test_mock_provider1: MockProvider,
    test_mock_provider2: MockProvider,
) -> None:
    """test_get.

    :param test_index_priority_strategy:
    :param test_mock_provider1:
    :param test_mock_provider2:
    """
    assert test_mock_provider1.get("test.test_key1", path_mode=True) == test_index_priority_strategy.get(
        "test.test_key1",
        path_mode=True,
    )
    assert test_mock_provider2.get("test.test_key4", path_mode=True) == test_index_priority_strategy.get(
        "test.test_key4",
        path_mode=True,
    )



from __future__ import annotations

import pytest

from sitri.strategy.index_priority import IndexPriorityStrategy

from .mock import MockProvider


@pytest.fixture(scope="module")
def test_mock_provider1() -> MockProvider:
    """test_mock_provider1."""
    return MockProvider(json_path="tests/unit/strategy/index_priority/data.json")


@pytest.fixture(scope="module")
def test_mock_provider2() -> MockProvider:
    """test_mock_provider2."""
    return MockProvider(json_path="tests/unit/strategy/index_priority/data_one.json")


@pytest.fixture
def test_index_priority_strategy(
    test_mock_provider1: MockProvider,
    test_mock_provider2: MockProvider,
) -> IndexPriorityStrategy:
    """test_index_priority_strategy.

    :param test_mock_provider1:
    :param test_mock_provider2:
    """
    return IndexPriorityStrategy(test_mock_provider1, test_mock_provider2)



"""
.. module:: providers.contrib.vault.__init__
   :synopsis: Contrib additional functional with vault
.. moduleauthor:: Aleksander Lavrov <github.com/egnod>
"""

from sitri.providers.contrib.vault.vault_kv import VaultKVConfigProvider



from __future__ import annotations

import pytest

from sitri.providers.contrib.system import SystemConfigProvider


@pytest.fixture(scope="module")
def system_config() -> SystemConfigProvider:
    """system_config.

    :rtype: SystemConfigProvider
    """
    return SystemConfigProvider(prefix="test")



from __future__ import annotations

import typing as t

import pytest
from pydantic.env_settings import SettingsError
from pydantic.error_wrappers import ValidationError

from sitri.providers.contrib.yaml import YamlConfigProvider
from sitri.settings.contrib.yaml import YamlSettings


def test_metadata(yaml_settings_empty: YamlSettings) -> None:
    """test_metadata.

    :param yaml_settings_empty:
    :rtype: None
    """
    assert yaml_settings_empty.Config.provider.provider_code == "yaml"


def test_get_variable(
    monkeypatch: t.Any,
    yaml_config: YamlConfigProvider,
    yaml_settings: t.Callable[[YamlConfigProvider], YamlSettings],
    yaml_settings_raise: t.Callable[[YamlConfigProvider], YamlSettings],
    yaml_settings_complex: t.Callable[[YamlConfigProvider], YamlSettings],
    yaml_settings_complex_raise: t.Callable[[YamlConfigProvider], YamlSettings],
) -> None:
    """test_get_variable.

    :param monkeypatch:
    :param yaml_config:
    :param yaml_settings:
    :param yaml_settings_raise:
    :param yaml_settings_complex:
    :param yaml_settings_complex_raise:
    :rtype: None
    """
    test_settings = yaml_settings(provider_instance=yaml_config)()

    assert test_settings.key1 == "1"
    assert test_settings.key2 == "2"
    assert test_settings.key3 == "3"

    test_settings_complex = yaml_settings_complex(provider_instance=yaml_config)()

    assert "test_key4_field" in test_settings_complex.key4.dict()
    assert test_settings_complex.key4.test_key4_field == "test"

    with pytest.raises(ValidationError):
        yaml_settings_raise(provider_instance=yaml_config)()

    with pytest.raises(SettingsError):
        yaml_settings_complex_raise(provider_instance=yaml_config)()



from __future__ import annotations

import typing as t

import vedis

from sitri.providers.base import ConfigProvider


class VedisConfigProvider(ConfigProvider):
    """Config provider for vedis storage."""

    provider_code = "vedis"
    _hash_name = "sitri_config_hash"

    def __init__(
        self,
        vedis_connector: t.Callable[[], vedis.Vedis],
        hash_name: str = "sitri_config_hash",
        *args: t.Any,
        **kwargs: t.Any,
    ) -> None:
        """

        :param vedis_connector: return connection to vedis
        :param hash_name: name for hash (key-value object) in vedis
        """
        super().__init__(*args, **kwargs)

        self._vedis_get = vedis_connector
        self._hash_name = hash_name
        self._vedis_instance = None

    @property
    def _vedis(self) -> vedis.Vedis:
        """_vedis.

        :rtype: vedis.Vedis
        """
        if not self._vedis_instance:
            self._vedis_instance = self._vedis_get()

        return self._vedis_instance

    @property
    def _config_hash(self) -> t.Any:
        """_config_hash."""
        return self._vedis.Hash(self._hash_name)

    def get(self, key: str, **kwargs: t.Any) -> str | None:
        """get.

        :param key:
        :type key: str
        :param kwargs:
        :rtype: t.Optional[str]
        """
        result = self._config_hash.get(key)

        if isinstance(result, bytes):
            return result.decode()

        return None

    def keys(self, **kwargs: t.Any) -> t.List[str]:
        """keys.

        :param kwargs:
        :rtype: t.List[str]
        """
        var_list = []
        variables = self._config_hash.keys() if self._config_hash.keys() is not None else []

        for var in variables:
            var_list.append(var.decode())

        return var_list



from __future__ import annotations

import json

import pytest

from sitri.providers.contrib.json import JsonConfigProvider


@pytest.fixture(scope="module")
def path_to_json() -> str:
    """path_to_json.

    :rtype: str
    """
    return "tests/unit/configs/json/data.json"


@pytest.fixture(scope="module")
def json_data() -> str:
    """json_data.

    :rtype: str
    """
    data = {"test": {"test_key1": "1", "test_key2": "2", "test_key3": "3"}}

    return json.dumps(data)


@pytest.fixture(scope="module")
def json_config_data(json_data: str) -> JsonConfigProvider:
    """json_config_data.

    :param json_data:
    :rtype: JsonConfigProvider
    """
    return JsonConfigProvider(json_data=json_data, default_separator="/")


@pytest.fixture(scope="module")
def json_config(path_to_json: str) -> JsonConfigProvider:
    """json_config.

    :param path_to_json:
    :rtype: JsonConfigProvider
    """
    return JsonConfigProvider(json_path=path_to_json, default_separator="/")



from __future__ import annotations

import pytest

from sitri.providers.contrib.yaml import YamlConfigProvider


def test_no_file_error() -> None:
    """test_no_file_error."""
    with pytest.raises(FileNotFoundError):
        YamlConfigProvider(yaml_path="_data.yaml")

    YamlConfigProvider(yaml_path="_data.yaml", found_file_error=False)


@pytest.mark.parametrize(
    "yaml_config_obj",
    [pytest.lazy_fixture("yaml_config"), pytest.lazy_fixture("yaml_data_config")],
)
def test_metadata(yaml_config_obj: YamlConfigProvider) -> None:
    """test_metadata.

    :param yaml_config_obj:
    :rtype: None
    """
    assert yaml_config_obj.provider_code == "yaml"
    assert yaml_config_obj.separator == "/"


@pytest.mark.parametrize(
    "yaml_config_obj",
    [pytest.lazy_fixture("yaml_config"), pytest.lazy_fixture("yaml_data_config")],
)
def test_get_by_other(yaml_config_obj: YamlConfigProvider) -> None:
    """test_get_by_other.

    :param yaml_config_obj:
    :rtype: None
    """
    assert isinstance(yaml_config_obj._get_by_path("test", separator=yaml_config_obj.separator), dict)
    assert yaml_config_obj._get_by_key("test.test_key2") is None
    assert yaml_config_obj._get_by_path("test.test_key2", separator=yaml_config_obj.separator) is None
    assert isinstance(yaml_config_obj._get_by_key("test"), dict)
    assert yaml_config_obj._get_by_path("test", separator=yaml_config_obj.separator) == yaml_config_obj._get_by_key(
        "test",
    )
    assert yaml_config_obj._get_by_path(
        "test/test_key2",
        separator=yaml_config_obj.separator,
    ) == yaml_config_obj._get_by_key("test").get("test_key2")


@pytest.mark.parametrize(
    "yaml_config_obj",
    [pytest.lazy_fixture("yaml_config"), pytest.lazy_fixture("yaml_data_config")],
)
def test_get(yaml_config_obj: YamlConfigProvider) -> None:
    """test_get.

    :param yaml_config_obj:
    """
    assert isinstance(yaml_config_obj.get("test"), dict)
    assert isinstance(yaml_config_obj.get("test", path_mode=True), dict)
    assert yaml_config_obj.get("test", path_mode=True) == yaml_config_obj.get("test")
    assert yaml_config_obj.get("test/test_key2", path_mode=True) and yaml_config_obj.get(
        "test/test_key2",
        path_mode=True,
    ) == yaml_config_obj.get("test").get("test_key2")
    assert yaml_config_obj.get("test.test_key2", separator=".", path_mode=True) == yaml_config_obj.get(
        "test/test_key2",
        path_mode=True,
    )
    assert not yaml_config_obj.get("test/test_key2")


@pytest.mark.parametrize(
    "yaml_config_obj",
    [pytest.lazy_fixture("yaml_config"), pytest.lazy_fixture("yaml_data_config")],
)
def test_keys(yaml_config_obj: YamlConfigProvider) -> None:
    """test_keys.

    :param yaml_config_obj:
    """
    assert "test" in yaml_config_obj.keys()

    with pytest.raises(NotImplementedError):
        yaml_config_obj.keys(path_mode=True)



from __future__ import annotations

import pytest

from sitri.providers.contrib.ini import IniConfigProvider


@pytest.fixture(scope="module")
def path_to_ini() -> str:
    """path_to_ini.

    :rtype: str
    """
    return "tests/unit/configs/ini/test.ini"


@pytest.fixture(scope="module")
def ini_config(path_to_ini: str) -> IniConfigProvider:
    """ini_config.

    :param path_to_ini:
    :rtype: IniConfigProvider
    """
    return IniConfigProvider(ini_path=path_to_ini)



"""
.. module:: config.providers
   :synopsis: Config Base
.. moduleauthor:: Aleksander Lavrov <github.com/egnod>
"""
from __future__ import annotations

import inspect
import typing as t
from abc import ABC, abstractmethod

from sitri.logger import get_default_logger


class ConfigProvider(ABC):
    """Base class for config providers."""

    def __init__(self, logger: t.Any | None = None, *args: t.Any, **kwargs: t.Any) -> None:
        """Default init method for all providers."""

        if not logger:
            logger = get_default_logger(__name__)

        self.logger = logger

    @property
    @abstractmethod
    def provider_code(self) -> str:
        """Provider code property for identity provider in manager."""

    @abstractmethod
    def get(self, key: str, **kwargs: t.Any) -> t.Any | None:
        """Get value from storage.

        :param key: key for find value in provider source
        :param kwargs: additional arguments for providers
        """

    @abstractmethod
    def keys(self, **kwargs: t.Any) -> t.List[str]:
        """Get keys list in storage."""

    def fill(self, call: t.Callable[[t.Any], t.Any], **kwargs: t.Any) -> t.Any:
        """Fill callable object kwargs if all founded by provider.

        :param call: callable object for fill
        :param kwargs: additional arguments for getting
        """
        parameters = inspect.signature(call).parameters
        data = {}

        for key in parameters.keys():
            data[key] = self.get(key=key, **kwargs)

        return call(**data)


class ConfigProviderManager:
    """Manager for children ConfigProvider classes."""

    @staticmethod
    def get_by_code(code: str) -> t.Type[ConfigProvider] | None:
        """Get config provider by provider_code.

        :param code: provider_code for search config provider
        :Example:
            .. code-block:: python

               ConfigProviderManager.get_by_code("system")
        """
        for provider in ConfigProvider.__subclasses__():
            if provider.provider_code == code:  # type: ignore
                return provider
        return None



from __future__ import annotations

import typing as t

import pytest

from sitri.providers.contrib.redis import RedisConfigProvider

from .mock import RedisMock


@pytest.fixture(scope="module")
def redis_connection() -> t.Callable[[], RedisMock]:
    """redis_connection.

    :rtype: t.Callable
    """
    return lambda: RedisMock()


@pytest.fixture(scope="module")
def redis_config(redis_connection: t.Callable[[], RedisMock]) -> RedisConfigProvider:
    """redis_config.

    :param redis_connection:
    :rtype: RedisConfigProvider
    """
    return RedisConfigProvider(prefix="test", redis_connector=redis_connection)



from __future__ import annotations

import pytest
import yaml

from sitri.providers.contrib.yaml import YamlConfigProvider

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO


@pytest.fixture(scope="module")
def yaml_data() -> str:
    """yaml_data.

    :rtype: str
    """
    data = {"test": {"test_key1": "1", "test_key2": "2", "test_key3": "3"}}

    stream = StringIO()

    yaml.dump(data, stream)

    return stream.getvalue()


@pytest.fixture(scope="module")
def path_to_yaml() -> str:
    """path_to_yaml.

    :rtype: str
    """
    return "tests/unit/configs/yaml/data.yaml"


@pytest.fixture(scope="module")
def yaml_data_config(yaml_data: str) -> YamlConfigProvider:
    """yaml_data_config.

    :param yaml_data:
    :rtype: YamlConfigProvider
    """
    return YamlConfigProvider(yaml_data=yaml_data, default_separator="/")


@pytest.fixture(scope="module")
def yaml_config(path_to_yaml: str) -> YamlConfigProvider:
    """yaml_config.

    :param path_to_yaml:
    :rtype: YamlConfigProvider
    """
    return YamlConfigProvider(yaml_path=path_to_yaml, default_separator="/")



from __future__ import annotations

import typing as t

from sitri.providers.base import ConfigProviderManager
from sitri.providers.contrib.json import JsonConfigProvider
from sitri.providers.contrib.redis import RedisConfigProvider
from sitri.providers.contrib.system import SystemConfigProvider
from sitri.providers.contrib.vedis import VedisConfigProvider


def test_get_by_codes(config_manager: t.Type[ConfigProviderManager]) -> None:
    """test_get_by_codes.

    :param config_manager:
    """
    assert config_manager.get_by_code(VedisConfigProvider.provider_code) == VedisConfigProvider
    assert config_manager.get_by_code(RedisConfigProvider.provider_code) == RedisConfigProvider
    assert config_manager.get_by_code(SystemConfigProvider.provider_code) == SystemConfigProvider
    assert config_manager.get_by_code(JsonConfigProvider.provider_code) == JsonConfigProvider



from __future__ import annotations

import typing as t

import hvac
from hvac.api.vault_api_base import VaultApiBase

from sitri.providers.base import ConfigProvider


class VaultKVConfigProvider(ConfigProvider):
    """Config provider for vault_kv storage."""

    provider_code = "vault_kv"

    def __init__(
        self,
        vault_connector: t.Callable[[], hvac.Client],
        mount_point: str | None = None,
        secret_path: str | None = None,
        version: int | None = 1,
        *args: t.Any,
        **kwargs: t.Any,
    ) -> None:
        """
        :param vault_connector: function return connection to Vault
        :param mount_point: default vault_kv storage mount point
        :param secret_path: default path to the secret in mounted
        """
        super().__init__(*args, **kwargs)
        assert version in (1, 2)

        self._vault_get = vault_connector
        self._vault_hvac_instance = None
        self._mount_point = mount_point
        self._secret_path = secret_path
        self._version = version

    @property
    def _vault(self) -> hvac.Client:
        """_vault.

        :rtype: hvac.Client
        """
        if not self._vault_hvac_instance:
            self._vault_hvac_instance = self._vault_get()

        return self._vault_hvac_instance

    @property
    def _vault_kv(self) -> VaultApiBase:
        return self._vault.secrets.kv.v1 if self._version == 1 else self._vault.secrets.kv.v2

    @property
    def _read_secret(self) -> t.Callable[[t.Any], t.Any]:
        return self._vault_kv.read_secret if self._version == 1 else self._vault_kv.read_secret_version

    def get(
        self, key: str, mount_point: str | None = None, secret_path: str | None = None, **kwargs: t.Any
    ) -> str | None:
        """get.

        :param key:
        :type key: str
        :param mount_point:
        :type mount_point: t.Optional[str]
        :param secret_path:
        :type secret_path: t.Optional[str]
        :param kwargs:
        :rtype: t.Optional[str]
        """
        request_params = {
            "path": secret_path if secret_path else self._secret_path,
            "mount_point": mount_point if mount_point else self._mount_point,
        }

        response = self._read_secret(**request_params)

        return response["data"].get(key)

    def keys(self, mount_point: str | None = None, secret_path: str | None = None, **kwargs: t.Any) -> t.List[str]:
        """keys.

        :param mount_point:
        :type mount_point: t.Optional[str]
        :param secret_path:
        :type secret_path: t.Optional[str]
        :param kwargs:
        :rtype: t.List[str]
        """
        request_params = {
            "path": secret_path if secret_path else self._secret_path,
            "mount_point": mount_point if mount_point else self._mount_point,
        }

        response = self._read_secret(**request_params)

        return list(response["data"].keys())



from __future__ import annotations

import typing as t

import pytest

from sitri.providers.base import ConfigProviderManager


@pytest.fixture(scope="module")
def config_manager() -> t.Type[ConfigProviderManager]:
    """config_manager.

    :rtype: t.Type[ConfigProviderManager]
    """
    return ConfigProviderManager



from __future__ import annotations

import typing as t

import hvac
import pytest
from pydantic import BaseModel, Field

from sitri.providers.contrib.vault import VaultKVConfigProvider
from sitri.settings.contrib.vault import VaultKVSettings

from .mock import VaultClientMock

T = t.TypeVar("T", bound=VaultKVSettings)


@pytest.fixture(scope="module")
def vault_connection() -> t.Callable[[], VaultClientMock]:
    """vault_connection.

    :rtype: t.Callable
    """
    return lambda: VaultClientMock()


@pytest.fixture(scope="module")
def vault_kv_config(vault_connection: t.Callable[[], hvac.Client]) -> VaultKVConfigProvider:
    """vault_kv_config.

    :param vault_connection:
    :rtype: VaultKVConfigProvider
    """
    return VaultKVConfigProvider(vault_connector=vault_connection, mount_point="test", secret_path="test")  # noqa


@pytest.fixture(scope="module")
def vault_kv_settings_empty(vault_kv_config: VaultKVConfigProvider) -> t.Type[T]:
    """vault_kv_settings_empty.

    :param vault_kv_config:
    """

    class TestSettings(VaultKVSettings):
        """TestSettings."""

        class Config(VaultKVSettings.VaultKVSettingsConfig):
            """Config."""

            provider = vault_kv_config

    return TestSettings


@pytest.fixture(scope="module")
def vault_kv_settings() -> t.Callable[[VaultKVConfigProvider], t.Type[T]]:
    """vault_kv_settings."""

    def wrapper(provider_instance: VaultKVConfigProvider) -> t.Type[T]:
        """wrapper.

        :param provider_instance:
        """

        class TestSettings(VaultKVSettings):
            """TestSettings."""

            key1: str = Field(...)
            key2: str = Field(...)

            class Config(VaultKVSettings.VaultKVSettingsConfig):
                """Config."""

                provider = provider_instance

        return TestSettings

    return wrapper


@pytest.fixture(scope="module")
def vault_kv_settings_vault_raise() -> t.Callable[[VaultKVConfigProvider], t.Type[T]]:
    """vault_kv_settings_vault_raise."""

    def wrapper(provider_instance: VaultKVConfigProvider) -> t.Type[T]:
        """wrapper.

        :param provider_instance:
        """

        class TestSettings(VaultKVSettings):
            """TestSettings."""

            key3: str = Field(...)

            class Config(VaultKVSettings.VaultKVSettingsConfig):
                """Config."""

                provider = provider_instance

        return TestSettings

    return wrapper


@pytest.fixture(scope="module")
def vault_kv_settings_complex() -> t.Callable[[VaultKVConfigProvider], t.Type[T]]:
    """vault_kv_settings_complex."""

    def wrapper(provider_instance: VaultKVConfigProvider) -> t.Type[T]:
        """wrapper.

        :param provider_instance:
        """

        class Key0Model(BaseModel):
            """Key0Model."""

            test: str = Field(...)

        class TestSettings(VaultKVSettings):
            """TestSettings."""

            key0: Key0Model = Field(...)

            class Config(VaultKVSettings.VaultKVSettingsConfig):
                """Config."""

                provider = provider_instance

        TestSettings.update_forward_refs(Key0Model=Key0Model)

        return TestSettings

    return wrapper


@pytest.fixture(scope="module")
def vault_kv_settings_complex_raise() -> t.Callable[[VaultKVConfigProvider], t.Type[T]]:
    """vault_kv_settings_complex_raise."""

    def wrapper(provider_instance: VaultKVConfigProvider) -> t.Type[T]:
        """wrapper.

        :param provider_instance:
        """

        class Key4Model(BaseModel):
            """Key4Model."""

            test: str = Field(...)

        class TestSettings(VaultKVSettings):
            """TestSettings."""

            key4: Key4Model = Field(...)

            class Config(VaultKVSettings.VaultKVSettingsConfig):
                """Config."""

                provider = provider_instance

        TestSettings.update_forward_refs(Key4Model=Key4Model)

        return TestSettings

    return wrapper


@pytest.fixture(scope="module")
def path_to_json() -> str:
    """path_to_json.

    :rtype: str
    """
    return "tests/unit/settings/vault_kv/data.json"


@pytest.fixture(scope="module")
def vault_kv_local_mode(path_to_json: str) -> t.Callable[[VaultKVConfigProvider], t.Type[T]]:
    """vault_kv_local_mode.

    :param path_to_json:
    """

    def wrapper(provider_instance: VaultKVConfigProvider) -> t.Type[T]:
        """wrapper.

        :param provider_instance:
        """

        class Key4Model(BaseModel):
            """Key4Model."""

            test: str = Field(...)

        class TestSettings(VaultKVSettings):
            """TestSettings."""

            key1: str = Field(...)
            key2: str = Field(...)
            key3: str = Field(...)
            key4: Key4Model = Field(...)

            class Config(VaultKVSettings.VaultKVSettingsConfig):
                """Config."""

                provider = provider_instance

                local_mode = True
                local_provider_args = {"json_path": path_to_json}
                local_mode_path_prefix = "test"

        TestSettings.update_forward_refs(Key4Model=Key4Model)

        return TestSettings

    return wrapper



from __future__ import annotations

import pytest
from _pytest.monkeypatch import MonkeyPatch

from sitri import Sitri
from sitri.configurator import SitriProviderConfigurator


def test_sitri_init() -> None:
    """test_sitri_init."""
    with pytest.raises(TypeError):
        Sitri()


def test_sitri_get_config(test_sitri: SitriProviderConfigurator, monkeypatch: MonkeyPatch) -> None:
    """test_sitri_get_config.

    :param test_sitri:
    :param monkeypatch:
    """
    monkeypatch.setenv("TEST_KEY1", "1")
    monkeypatch.setenv("TEST_KEY2", "2")

    assert test_sitri.get("key1") == "1"
    assert test_sitri.get("key2") == "2"
    assert not test_sitri.get("key3")

    monkeypatch.undo()
